<?php

/*

    番組表関係

*/


// $url を取得
// ステータスコードを返す
// $contents に内容が入る
function get_webpage( $url, &$contents, &$response_header )
{
    $timeout = TIMEOUT_GET;
    $proxy = PROXY;
    $referer = '';

    $ua = '';
    if( isset( $_SERVER[ 'HTTP_USER_AGENT' ] ) ) $ua = $_SERVER[ 'HTTP_USER_AGENT' ];

    $header = '';
    if( $ua ) $header = $header.'User-Agent: '.$ua."\r\n";
    if( $referer ) $header = $header.'Referer: '.$referer."\r\n";

    show_debug( 'get_webpage url = '.$url );
    show_debug( 'header = '.$header );

    list( $protcol, $path ) = explode( '//', $url, 2 );
    show_debug( 'protcol = '.$protcol );
    list( $host, $path ) = explode( '/', $path, 2 );
    show_debug( 'host = '.$host );
    show_debug( 'path = '.$path );

    // ローカルホストの場合はプロキシ無効
    if( $host == 'localhost' || $host == '127.0.0.1' ) $proxy = '';

    $options = array(

        'http' => array(
            'method' => 'GET',
            'timeout' => $timeout,
            'header' => $header,
            )
        );

    // プロキシ
    if( $proxy ){

        show_debug( 'proxy = '.$proxy );

        $options[ 'http' ][ 'proxy' ] = $proxy;
        $options[ 'http' ][ 'request_fulluri' ] = TRUE;
    }

    $contents = file_get_contents( $url, FALSE, stream_context_create( $options ) );
    if( $contents == FALSE ) throw new Exception( $url.' の読み込みに失敗しました' );

    list( $http, $code, $msg ) = explode( ' ', $http_response_header[ 0 ], 3 );
    $response_header = $http_response_header;

    return $code;
}


// キャッシュディレクトリ内にある古いファイルを削除する
// $type == 'all' の時は更新時刻をチェックしないで全て消す
function remove_oldcache( $type )
{
    show_debug( 'remove_oldcache type = '.$type );

    $duration = CACHEDAY * 3600 * 24;
    show_debug( 'duration = '.$duration );

    $files = scandir( get_cachedir() );
    foreach( $files as $file ){

        if( $file == '.' || $file == '..' ) continue;

        $path = get_cachedir().$file;

        if( $type == 'all' ){
            show_debug( 'remove '.$path );
            unlink( $path );
            continue;
        }

        if( ! ( $fp = fopen( $path, 'r') ) ){
            throw new Exception(
                'エラー(remove_oldcache) :ファイル('.$path.')のオープンに失敗しました。パーミッションを確認してください。' );
        }
        $stat = fstat( $fp );
        fclose( $fp );

        $time = time()-$stat[ 'mtime' ];
        if( $time >= $duration ){
            show_debug( 'remove time = '.$time.' '.$path );
            unlink( $path );
        }
    }
}


// TV王国から番組表をダウンロードして変換
function get_prog_table( &$table, $area, $station, $year, $month, $day, $hour )
{
    $time = mktime( $hour, 0, 0, $month, $day, $year );
    $url = TVOKOKU.TVOKOKUCHART.$area.'.action?span=6&chartWidth=1200&cellHeight=5&head='.date( 'YmdH00', $time );
    show_debug( 'get_prog_table url = '.$url );

    // キャッシュが存在するならキャッシュを使う
    $path_cache = get_cachedir().'cache-'.$area.'-'.$station.'-'.$year.'-'.$month.'-'.$day.'-'.$hour;
    if( file_exists( $path_cache ) ){

        show_debug( 'cache exists path = '.$path_cache );
        $table = file_get_contents( $path_cache );
        return;
    }

    // ダウンロード
    $contents = '';
    $response_header = false;
    $code = get_webpage( $url, $contents, $response_header );
    if( $code != 200 ) throw new Exception( $response_header[ 0 ] );
    show_debug( 'http code='.$code );

    // PCRE のバックトラック処理の制限値を上げる
    // デフォルトが 100000 なので番組表のサイズが大きいと preg_match が失敗する
    ini_set( 'pcre.backtrack_limit', BACKTRACK_LIMIT );

    $table = '';
    if( preg_match( '/<div *id="chartColumn" *class="chartColumn".*?chartColumn-->/s', $contents, $reg ) ){

        show_debug( 'found body' );

        $table =  '<div style="position:relative;">'.$reg[ 0 ].'</div></div>';

        // 前、次の時間帯へを削除
        $table = preg_replace( '/<a *href="\/chart[^>]*?prev-chart" *>.*?<\/a>/s', '', $table );
        $table = preg_replace( '/<a *href="\/chart[^>]*?next-chart" *>.*?<\/a>/s', '', $table );

        // 局切り替えを削除
        $table = preg_replace( '/<a *class="inlineArrow[^>]*>.*?<\/a>/s', '', $table );

        // 表内で使用するアイコンをダウンロードしてキャッシュに入れておく
        $icondir = TVOKOKUICON;
        $icondir = str_replace( '/', '\/', $icondir );
        preg_match_all( '/'.$icondir.'([^"]*)"/', $table, $tmpmatch );
        $icons = array_unique( $tmpmatch[ 1 ] );
        foreach( $icons as $icon ){

            $path = get_cachedir().$icon;
            if( ! file_exists( $path  ) ){

                $url_icon = TVOKOKU.TVOKOKUICON.$icon;

                show_debug( 'download : '.$url_icon.' -> '.$path );

                $contents_icon = '';
                $response_header = false;
                $code = get_webpage( $url_icon, $contents_icon, $response_header );
                if( $code != 200 ) throw new Exception( $response_header[ 0 ] );

                file_put_contents( $path, $contents_icon );
            }
        }

        // リファラが外に出るとセキュリティ上問題があるので番組情報のリンクを文字列にする
        $table = preg_replace( '/<a href="\/schedule[^>]*>(.*?)<\/a>/s', '\\1', $table );

        // 「気になる」削除
        $table = preg_replace( '/<span *class="kininaruIcon".*?<\/span>/s', '', $table );

        // onmouseover 削除
        $table = preg_replace( '/onmouseover="[^"]*?"/', '', $table );

        // onmouseout 削除
        $table = preg_replace( '/onmouseout="[^"]*?"/', '', $table );

        // window.onscroll 削除
        $table = preg_replace( '/window\.onscroll *= *[^;]*;/', '', $table );

        // SimpleChart. 削除
        $table = preg_replace( '/SimpleChart\./', '', $table );

        // stickyHeader.js 削除
        $table = preg_replace( '/<script[^>]*stickyHeader\.js" *> *<\/script>/', '', $table );

        // アイコンのアドレス置き換え
        $table = preg_replace( '/'.$icondir.'/', './'.CACHEDIR.'/', $table );

        // iEPGのdivを削除
        $table = preg_replace( '/<div id="iepg.*?>iEPG<\/div>/', 'iEPG', $table ); 

        // iEPG のアドレス変更
        $url_back = rawurlencode( './tvtable.php?area='.$area.'&year='.$year.'&month='.$month.'&day='.$day.'&hour='.$hour );
        $url_iepg = rawurlencode( TVOKOKU.'/iepg.tvpi?id=' );
        $table = preg_replace( '/\/iepg.tvpi\?id=([0-9]*)/', './reservedit.php?iepg='.$url_iepg.'\\1&back='.$url_back, $table );
    }
    else{

        if( preg_last_error() == PREG_BACKTRACK_LIMIT_ERROR )
            throw new Exception( '正規表現のバックトラック処理の制限値を越えました' );

        if( preg_last_error() == PREG_RECURSION_LIMIT_ERROR )
            throw new Exception( '正規表現の再帰処理の制限量を越えました' );

        throw new Exception( '番組テーブルの取得に失敗しました' );
    }

    // セルクリックで開閉のJavaScript
    $jscript = '<script language="JavaScript">'.
    "<!--\n".

    "var precell = 0;".

    "        function shrinkCell( id )".
    "        {".
    "            var cell = document.getElementById( id );".

    "            if( cell.style.zIndex != 40 ){".

    "                if( precell != 0 )".
    "                {".
    "                    precell.style.width = precell.originalWidth;".
    "                    precell.style.height = precell.originalHeight;".
    "                    precell.style.zIndex = 1;".
    "                    precell = 0;".
    "                }".

    "                cell.originalWidth = cell.style.width;".
    "                cell.originalHeight = cell.style.height;".
    "                if (cell.scrollHeight > cell.offsetHeight)".
    "                {".
    "                    cell.style.Width  = cell.scrollWidth + 'px';".
    "                    cell.style.height = cell.scrollHeight + 'px';".
    "                    cell.style.zIndex = 40;".
    "                    precell = cell;".
    "                }".
    "            }".
    "            else{".
    "                cell.style.width = cell.originalWidth;".
    "                cell.style.height = cell.originalHeight;".
    "                cell.style.zIndex = 1;".
    "                precell = 0;".
    "            }".
    "        }\n".

    '//-->'.
    '</script>'
    ;

    $table = $jscript.$table;

    file_put_contents( $path_cache, $table );
    show_debug( 'save cache path = '.$path_cache );
}


// iEPG 取得
function get_iepg( $url, &$setting, &$iepg, &$time )
{
    global $CHANNEL_TABLE;

    $contents = '';
    $response_header = false;
    $code = get_webpage( $url, $contents, $response_header );
    if( $code != 200 ) throw new Exception( $response_header[ 0 ] );

    // 解析
    $station = FALSE;
    $year = 0;
    $month = 0;
    $day = 0;
    $hour_start = 0;
    $min_start = 0;
    $hour_end = 0;
    $min_end = 0;

    $iepg = mb_convert_encoding( $contents, "UTF-8", "SJIS" );
    $row = explode( "\n", $iepg );

    foreach( $row as $line ){

        $item = explode( ':', $line, 2 );
        if( count( $item ) == 2 && $item[ 0 ] && $item[ 1 ] ){

            show_debug( $item[ 0 ].' : '.$item[ 1 ] );

            if( $item[ 0 ] == 'station' ){

                 // 局名は全角英数字を半角にする
                 $station = trim( mb_convert_kana( $item[ 1 ], "a", "UTF-8" ) );
            }
            if( $item[ 0 ] == 'program-title' ){

                $setting[ SET_COMMENT ] = trim( $item[ 1 ] );
                $setting[ SET_NAMEBASE ] = trim( $item[ 1 ] );
            }
            if( $item[ 0 ] == 'year' ) $year = (int)$item[ 1 ];
            if( $item[ 0 ] == 'month' ) $month = (int)$item[ 1 ];
            if( $item[ 0 ] == 'date' )  $day = (int)$item[ 1 ];
            if( $item[ 0 ] == 'start' ){

                $tmp = explode( ':', $item[ 1 ] );
                $hour_start = (int)$tmp[ 0 ];
                $min_start = (int)$tmp[ 1 ];
            }
            if( $item[ 0 ] == 'end' ){

                $tmp = explode( ':', $item[ 1 ] );
                $hour_end = (int)$tmp[ 0 ];
                $min_end = (int)$tmp[ 1 ];
            }
        }
    }

    // 時刻取得
    $time = mktime( $hour_start, $min_start, 0, $month, $day, $year );
    $time_end = mktime( $hour_end, $min_end, 0, $month, $day, $year );
    if( $time_end <= $time ) $time_end += 3600*24;

    $setting[ SET_DURATION ] = $time_end - $time;

    // チャンネル取得
    foreach( $CHANNEL_TABLE as $ch => $chname ){

        $tmpname = mb_convert_kana( $chname, "a", "UTF-8" );
        show_debug( $station.'/'.$tmpname.' ch='.$ch );

        if( $station == $tmpname ){
            show_debug( 'set ch='.$ch );
            $setting[ SET_CHANNEL ] = $ch;
            break;
        }
    }

}


?>